use core::{cell::UnsafeCell, mem::ManuallyDrop};

pub fn unit<T>(_: T) {}

pub struct DropGuard<F: FnOnce()>(UnsafeCell<ManuallyDrop<F>>);

impl<F> DropGuard<F>
where
    F: FnOnce(),
{
    pub fn new(f: F) -> DropGuard<F> {
        Self(UnsafeCell::new(ManuallyDrop::new(f)))
    }
}

impl<F> Drop for DropGuard<F>
where
    F: FnOnce(),
{
    fn drop(&mut self) {
        unsafe {
            ManuallyDrop::take(&mut *self.0.get())();
        }
    }
}
